/**
 *  This file is part of DJFourierv4 developed by Blai Meléndez Catalán under
 	the supervision of prof. Horst Eidenberger (TU Wien).

    DJFourierv4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DJFourierv4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

 	<http://www.gnu.org/licenses/>. */

package com.example.djfourierv4;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import android.util.Log;

public class WavReader {
		
	private static final int HEADER_SIZE = 44;
		
	private ByteBuffer bufferHeader;
	
	private boolean finish = false;
				
	public WavReader(){	
	}
	
	/**
	 * Reads the header of the wave file and returns the info inside an object of class WavInfo */
	public WavInfo readHeader(byte[] wavBytes) throws IOException {
			 
		bufferHeader = ByteBuffer.allocate(HEADER_SIZE);
			 		
		bufferHeader = ByteBuffer.wrap(wavBytes);
		
		bufferHeader.order(ByteOrder.LITTLE_ENDIAN);
		
		bufferHeader.rewind();
		bufferHeader.position(20);
		
		int format = bufferHeader.getShort();

		if(format != 1){
			finish = true;
		}
		
		int channels = bufferHeader.getShort();
		Log.d("channels", "Number of channels = " + channels);
		if(channels != 1){
			finish = true;
			Log.d("finish", "finish = " + finish);
		}

		int rate = bufferHeader.getInt();		
		
		bufferHeader.position(bufferHeader.position() + 6);
		
		int bits = bufferHeader.getShort();
		
		if(bits != 8 && bits != 16){
			finish = true;
		}
		
		bufferHeader.position(bufferHeader.position() + 4);
				
		return new WavInfo(format, channels, rate, bits, 0, finish);
	}	
}
