/**
 *  This file is part of DJFourierv4 developed by Blai Meléndez Catalán under
 	the supervision of prof. Horst Eidenberger (TU Wien).

    DJFourierv4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DJFourierv4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

 	<http://www.gnu.org/licenses/>. */

package com.example.djfourierv4;

public class SpectrumManager {
	
	private static int IR = 8;

	public SpectrumManager(){
		
	}
	
	public static double[] createParabolas(double[] data, double[] maxPos){
		
		double[] result = new double[IR * data.length];
		
		//Create the space for the interpolation
		for(int j = 0; j < data.length; j++){
			
			result[IR * j] = data[j];
		}
		
		//Interpolation
		for(int i = 0; i < maxPos.length; i++){
			
			//only for the first 35 maxs
			if(maxPos[i] >= 35){
				break;
			}
			
			//If there is a maximum at 0
			if(maxPos[i] == 0){
				
				//Gaussian parameters
				double maxBinNum = 0;
				double a = (Math.log(data[(int) maxPos[i]]) - Math.log(data[(int)maxPos[i] + 1])) / (Math.pow(maxPos[i] - maxBinNum, 2) - Math.pow(maxPos[i] + 1 - maxBinNum, 2));
				double h = Math.log(data[(int) maxPos[i] + 1]) - a * Math.pow(maxPos[i] + 1 - maxBinNum, 2);
				
				//Filling the array
				for(int k = 1; k < IR; k++){
					result[(IR * (int) maxPos[i] + k)] = Math.pow(Math.E, a * Math.pow((maxPos[i] + (double) k / (double) IR) - maxBinNum, 2) + h);
				}

				
			}
			
			//If there is a maximum at the end of the spectrum
			if(maxPos[i] == data.length - 1){
				
				//Gaussian parameters
				double maxBinNum = data.length - 1;
				double a = (Math.log(data[(int) maxPos[i]]) - Math.log(data[(int) (maxPos[i] - 1)])) / (Math.pow(maxPos[i] - maxBinNum, 2) - Math.pow(maxPos[i] - 1 - maxBinNum, 2));
				double h = Math.log(data[(int) (maxPos[i] - 1)]) - a * Math.pow(maxPos[i] - 1 - maxBinNum, 2);
				
				//Filling the array
				for(int k = 1; k > -IR; k--){
					result[(int) (IR * maxPos[i] + k)] = Math.pow(Math.E, a * Math.pow((maxPos[i] + (double) k / (double) IR) - maxBinNum, 2) + h);
				}
				
			}
			
			//For maxima in the middle of the spectrum
			if(maxPos[i] != 0 && maxPos[i] != data.length - 1){
				
				//Gaussian parameters
				double maxBinNum = maxPos[i] + (Math.log(data[(int) (maxPos[i] + 1)] / data[(int) (maxPos[i] - 1)]) / (2 * Math.log(Math.pow(data[(int) maxPos[i]], 2) / (data[(int) (maxPos[i] + 1)] * data[(int) (maxPos[i] - 1)]))));
				double a = (Math.log(data[(int) maxPos[i]]) - Math.log(data[(int) (maxPos[i] - 1)])) / (Math.pow(maxPos[i] - maxBinNum, 2) - Math.pow(maxPos[i] - 1 - maxBinNum, 2));
				double h = Math.log(data[(int) (maxPos[i] - 1)]) - a * Math.pow(maxPos[i] - 1 - maxBinNum, 2);
				
				//Filling the array
				for(int k = 1; k < IR; k++){
					result[(int) (IR * maxPos[i] + k)] = Math.pow(Math.E, a * Math.pow((maxPos[i] + (double) k / (double) IR) - maxBinNum, 2) + h);
				}
				
				for(int k = -1; k > -IR; k--){
					result[(int) (IR * maxPos[i] + k)] = Math.pow(Math.E, a * Math.pow((maxPos[i] + (double) k / (double) IR) - maxBinNum, 2) + h);
				}
			}
		}
		
		return result;
	}
}
